@@ -38,6 +38,7 @@
 #include "got_commit_graph.h"
 #include "got_blame.h"
 #include "got_privsep.h"
+#include "got_opentemp.h"
 
 #include "gotwebd.h"
 #include "log.h"
@@ -57,12 +58,7 @@ got_gotweb_closefile(FILE *f)
 {
 	const struct got_error *err = NULL;
 
-	if (fseek(f, 0, SEEK_SET) == -1)
-		err = got_error_from_errno("fseek");
-
-	if (ftruncate(fileno(f), 0) == -1 && err == NULL)
-		err = got_error_from_errno("ftruncate");
-
+	err = got_opentemp_truncate(f);
 	if (fclose(f) == EOF && err == NULL)
 		err = got_error_from_errno("fclose");
 
@@ -333,12 +329,10 @@ got_get_repo_commits(struct request *c, size_t limit)
 	struct got_reflist_head refs;
 	struct got_reference *ref = NULL;
 	struct repo_commit *repo_commit = NULL;
-	struct server *srv = c->srv;
 	struct transport *t = c->t;
 	struct got_repository *repo = t->repo;
 	struct querystring *qs = t->qs;
-	struct repo_dir *repo_dir = t->repo_dir;
-	char *in_repo_path = NULL, *repo_path = NULL, *file_path = NULL;
+	char *file_path = NULL;
 	int chk_next = 0;
 
 	if (limit == 0)
@@ -360,12 +354,6 @@ got_get_repo_commits(struct request *c, size_t limit)
 		    qs->file) == -1)
 			return got_error_from_errno("asprintf");
 
-	if (asprintf(&repo_path, "%s/%s", srv->repos_path,
-	    repo_dir->name) == -1) {
-		error = got_error_from_errno("asprintf");
-		goto done;
-	}
-
 	if (qs->commit) {
 		error = got_repo_match_object_id_prefix(&id, qs->commit,
 		    GOT_OBJ_TYPE_COMMIT, repo);
@@ -381,10 +369,6 @@ got_get_repo_commits(struct request *c, size_t limit)
 			goto done;
 	}
 
-	error = got_repo_map_path(&in_repo_path, repo, repo_path);
-	if (error)
-		goto done;
-
 	error = got_ref_list(&refs, repo, NULL, got_ref_cmp_by_name, NULL);
 	if (error)
 		goto done;
@@ -394,7 +378,7 @@ got_get_repo_commits(struct request *c, size_t limit)
 		if (error)
 			goto done;
 	} else {
-		error = got_commit_graph_open(&graph, in_repo_path, 0);
+		error = got_commit_graph_open(&graph, "/", 0);
 		if (error)
 			goto done;
 	}
@@ -455,9 +439,7 @@ got_get_repo_commits(struct request *c, size_t limit)
 	if (graph)
 		got_commit_graph_close(graph);
 	got_ref_list_free(&refs);
-	free(in_repo_path);
 	free(file_path);
-	free(repo_path);
 	free(id);
 	return error;
 }
@@ -477,7 +459,7 @@ got_get_repo_tags(struct request *c, size_t limit)
 	struct querystring *qs = t->qs;
 	struct repo_dir *repo_dir = t->repo_dir;
 	struct got_tag_object *tag = NULL;
-	char *in_repo_path = NULL, *repo_path = NULL, *id_str = NULL;
+	char *repo_path = NULL, *id_str = NULL;
 	char *tag_commit = NULL, *tag_commit0 = NULL;
 	char *commit_msg = NULL, *commit_msg0 = NULL;
 	int chk_next = 0, chk_multi = 1, commit_found = 0;
@@ -522,10 +504,6 @@ got_get_repo_tags(struct request *c, size_t limit)
 		}
 	}
 
-	error = got_repo_map_path(&in_repo_path, repo, repo_path);
-	if (error)
-		goto done;
-
 	error = got_ref_list(&refs, repo, "refs/tags", got_ref_cmp_tags,
 	   repo);
 	if (error)
@@ -682,7 +660,6 @@ got_get_repo_tags(struct request *c, size_t limit)
 		got_object_tag_close(tag);
 	got_ref_list_free(&refs);
 	free(commit_msg0);
-	free(in_repo_path);
 	free(repo_path);
 	free(id);
 	free(id_str);
@@ -703,10 +680,9 @@ got_output_repo_tree(struct request *c, char **readme,
 	struct got_reflist_head refs;
 	struct got_tree_object *tree = NULL;
 	struct got_tree_entry *te;
-	struct repo_dir *repo_dir = t->repo_dir;
 	const char *name;
 	mode_t mode;
-	char *escaped_name = NULL, *path = NULL;
+	char *escaped_name = NULL;
 	int nentries, i;
 
 	TAILQ_INIT(&refs);
@@ -714,18 +690,6 @@ got_output_repo_tree(struct request *c, char **readme,
 
 	rc = TAILQ_FIRST(&t->repo_commits);
 
-	if (qs->folder != NULL) {
-		path = strdup(qs->folder);
-		if (path == NULL) {
-			error = got_error_from_errno("strdup");
-			goto done;
-		}
-	} else {
-		error = got_repo_map_path(&path, repo, repo_dir->path);
-		if (error)
-			goto done;
-	}
-
 	error = got_repo_match_object_id(&commit_id, NULL, rc->commit_id,
 	    GOT_OBJ_TYPE_COMMIT, &refs, repo);
 	if (error)
@@ -735,7 +699,8 @@ got_output_repo_tree(struct request *c, char **readme,
 	if (error)
 		goto done;
 
-	error = got_object_id_by_path(&tree_id, repo, commit, path);
+	error = got_object_id_by_path(&tree_id, repo, commit,
+	    qs->folder ? qs->folder : "/");
 	if (error)
 		goto done;
 
@@ -764,7 +729,6 @@ got_output_repo_tree(struct request *c, char **readme,
 	}
  done:
 	free(escaped_name);
-	free(path);
 	got_ref_list_free(&refs);
 	if (commit)
 		got_object_commit_close(commit);
@@ -793,7 +757,7 @@ got_open_blob_for_output(struct got_blob_object **blob, int *fd,
 	struct got_object_id *commit_id = NULL;
 	struct got_object_id *blob_id = NULL;
 	struct got_reflist_head refs;
-	char *path = NULL, *in_repo_path = NULL;
+	char *path = NULL;
 	int obj_type;
 
 	TAILQ_INIT(&refs);
@@ -813,10 +777,6 @@ got_open_blob_for_output(struct got_blob_object **blob, int *fd,
 		goto done;
 	}
 
-	error = got_repo_map_path(&in_repo_path, repo, path);
-	if (error)
-		goto done;
-
 	if (commitstr == NULL)
 		commitstr = GOT_REF_HEAD;
 
@@ -829,7 +789,7 @@ got_open_blob_for_output(struct got_blob_object **blob, int *fd,
 	if (error)
 		goto done;
 
-	error = got_object_id_by_path(&blob_id, repo, commit, in_repo_path);
+	error = got_object_id_by_path(&blob_id, repo, commit, path);
 	if (error)
 		goto done;
 
@@ -873,7 +833,6 @@ got_open_blob_for_output(struct got_blob_object **blob, int *fd,
 	}
 
 	got_ref_list_free(&refs);
-	free(in_repo_path);
 	free(commit_id);
 	free(blob_id);
 	free(path);
@@ -1012,7 +971,7 @@ got_output_file_blame(struct request *c, got_render_blame_line_cb cb)
 	struct got_commit_object *commit = NULL;
 	struct got_reflist_head refs;
 	struct got_blob_object *blob = NULL;
-	char *path = NULL, *in_repo_path = NULL;
+	char *path = NULL;
 	struct blame_cb_args bca;
 	int i, obj_type, blobfd = -1, fd1 = -1, fd2 = -1;
 	off_t filesize;
@@ -1028,10 +987,6 @@ got_output_file_blame(struct request *c, got_render_blame_line_cb cb)
 		goto done;
 	}
 
-	error = got_repo_map_path(&in_repo_path, repo, path);
-	if (error)
-		goto done;
-
 	error = got_repo_match_object_id(&commit_id, NULL, qs->commit,
 	    GOT_OBJ_TYPE_COMMIT, &refs, repo);
 	if (error)
@@ -1041,7 +996,7 @@ got_output_file_blame(struct request *c, got_render_blame_line_cb cb)
 	if (error)
 		goto done;
 
-	error = got_object_id_by_path(&obj_id, repo, commit, in_repo_path);
+	error = got_object_id_by_path(&obj_id, repo, commit, path);
 	if (error)
 		goto done;
 
@@ -1106,7 +1061,7 @@ got_output_file_blame(struct request *c, got_render_blame_line_cb cb)
 	if (error)
 		goto done;
 
-	error = got_blame(in_repo_path, commit_id, repo,
+	error = got_blame(path, commit_id, repo,
 	    GOT_DIFF_ALGORITHM_PATIENCE, got_gotweb_blame_cb, &bca, NULL, NULL,
 	    fd1, fd2, f1, f2);
 
@@ -1145,7 +1100,6 @@ got_output_file_blame(struct request *c, got_render_blame_line_cb cb)
 		got_object_commit_close(commit);
 	if (blob)
 		got_object_blob_close(blob);
-	free(in_repo_path);
 	free(commit_id);
 	free(obj_id);
 	free(path);
