@@ -574,9 +574,6 @@ connect_children(struct gotwebd *env)
 
 		if (send_imsg(iev2, GOTWEBD_IMSG_CTL_PIPE, pipe[1], NULL, 0))
 			fatal("send_imsg");
-
-		close(pipe[0]);
-		close(pipe[1]);
 	}
 }
 
@@ -689,7 +686,6 @@ gotwebd_shutdown(void)
 		TAILQ_REMOVE(&gotwebd_env->servers, srv, entry);
 		free(srv);
 	}
-	sockets_purge(gotwebd_env);
 	free(gotwebd_env);
 
 	log_warnx("gotwebd terminating");
