@@ -32,7 +32,11 @@
 #define GOTWEBD_CONF		 "/etc/gotwebd.conf"
 
 #ifndef GOTWEBD_DEFAULT_USER
-#define GOTWEBD_DEFAULT_USER	 "www"
+#define GOTWEBD_DEFAULT_USER	 "_gotwebd"
+#endif
+
+#ifndef GOTWEBD_WWW_USER
+#define GOTWEBD_WWW_USER	 "www"
 #endif
 
 #define GOTWEBD_MAXDESCRSZ	 1024
@@ -117,12 +121,20 @@ struct got_blob_object;
 struct got_tree_entry;
 struct got_reflist_head;
 
+enum gotwebd_proc_type {
+	GOTWEBD_PROC_PARENT,
+	GOTWEBD_PROC_SERVER,
+	GOTWEBD_PROC_GOTWEB,
+};
+
 enum imsg_type {
-	IMSG_CFG_SRV,
-	IMSG_CFG_SOCK,
-	IMSG_CFG_FD,
-	IMSG_CFG_DONE,
-	IMSG_CTL_START,
+	GOTWEBD_IMSG_CFG_SRV,
+	GOTWEBD_IMSG_CFG_SOCK,
+	GOTWEBD_IMSG_CFG_FD,
+	GOTWEBD_IMSG_CFG_DONE,
+	GOTWEBD_IMSG_CTL_PIPE,
+	GOTWEBD_IMSG_CTL_START,
+	GOTWEBD_IMSG_REQ_PROCESS,
 };
 
 struct imsgev {
@@ -229,6 +241,7 @@ enum socket_priv_fds {
 
 struct template;
 struct request {
+	TAILQ_ENTRY(request)		entry;
 	struct socket			*sock;
 	struct server			*srv;
 	struct transport		*t;
@@ -239,12 +252,15 @@ struct request {
 	uint16_t			 id;
 	int				 fd;
 	int				 priv_fd[PRIV_FDS__MAX];
+	struct event			*resp_event;
+	int				 sock_id;
+	uint32_t			 request_id;
 
-	uint8_t				 buf[FCGI_RECORD_SIZE];
+	uint8_t				 *buf;
 	size_t				 buf_pos;
 	size_t				 buf_len;
 
-	uint8_t				 outbuf[GOTWEBD_CACHESIZE];
+	uint8_t				 *outbuf;
 
 	char				 querystring[MAX_QUERYSTRING];
 	char				 document_uri[MAX_DOCUMENT_URI];
@@ -252,7 +268,9 @@ struct request {
 	int				 https;
 
 	uint8_t				 request_started;
+	int				 client_status;
 };
+TAILQ_HEAD(requestlist, request);
 
 struct fcgi_begin_request_body {
 	uint16_t	role;
@@ -307,6 +325,7 @@ TAILQ_HEAD(serverlist, server);
 
 enum client_action {
 	CLIENT_CONNECT,
+	CLIENT_REQUEST,
 	CLIENT_DISCONNECT,
 };
 
@@ -327,8 +346,6 @@ struct socket {
 	struct event	 evt;
 	struct event	 ev;
 	struct event	 pause;
-
-	int		 client_status;
 };
 TAILQ_HEAD(socketlist, socket);
 
@@ -342,6 +359,7 @@ struct gotwebd {
 	int		 priv_fd[PRIV_FDS__MAX];
 
 	char		*user;
+	char		*www_user;
 	const char	*gotwebd_conffile;
 
 	int		 gotwebd_debug;
@@ -349,12 +367,12 @@ struct gotwebd {
 
 	struct imsgev	*iev_parent;
 	struct imsgev	*iev_server;
+	struct imsgev	*iev_gotweb;
 	size_t		 nserver;
 
-	struct passwd	*pw;
-
 	uint16_t	 prefork_gotwebd;
-	int		 gotwebd_reload;
+	int		 servers_pending;
+	int		 gotweb_pending;
 
 	int		 server_cnt;
 
@@ -430,17 +448,20 @@ typedef int (*got_render_blame_line_cb)(struct template *, const char *,
 /* gotwebd.c */
 void	 imsg_event_add(struct imsgev *);
 int	 imsg_compose_event(struct imsgev *, uint16_t, uint32_t,
-	    pid_t, int, const void *, uint16_t);
+	    pid_t, int, const void *, size_t);
 int	 main_compose_sockets(struct gotwebd *, uint32_t, int,
 	    const void *, uint16_t);
 int	 sockets_compose_main(struct gotwebd *, uint32_t,
 	    const void *, uint16_t);
+int	 main_compose_gotweb(struct gotwebd *, uint32_t, int,
+	    const void *, uint16_t);
 
 /* sockets.c */
 void sockets(struct gotwebd *, int);
 void sockets_parse_sockets(struct gotwebd *);
 void sockets_socket_accept(int, short, void *);
-int sockets_privinit(struct gotwebd *, struct socket *);
+int sockets_privinit(struct gotwebd *, struct socket *, uid_t, gid_t);
+void sockets_rlimit(int);
 
 /* gotweb.c */
 void gotweb_index_navs(struct request *, struct gotweb_url *, int *,
@@ -452,8 +473,9 @@ int gotweb_render_url(struct request *, struct gotweb_url *);
 int gotweb_render_absolute_url(struct request *, struct gotweb_url *);
 void gotweb_free_repo_commit(struct repo_commit *);
 void gotweb_free_repo_tag(struct repo_tag *);
-void gotweb_process_request(struct request *);
+int gotweb_process_request(struct request *);
 void gotweb_free_transport(struct transport *);
+void gotweb(struct gotwebd *, int);
 
 /* pages.tmpl */
 int	gotweb_render_page(struct template *, int (*)(struct template *));
@@ -483,7 +505,6 @@ void fcgi_request(int, short, void *);
 void fcgi_timeout(int, short, void *);
 void fcgi_cleanup_request(struct request *);
 void fcgi_create_end_record(struct request *);
-void dump_fcgi_record(const char *, struct fcgi_record_header *);
 int fcgi_write(void *, const void *, size_t);
 
 /* got_operations.c */
@@ -507,7 +528,7 @@ const struct got_error *got_output_file_blame(struct request *,
 /* config.c */
 int config_setserver(struct gotwebd *, struct server *);
 int config_getserver(struct gotwebd *, struct imsg *);
-int config_setsock(struct gotwebd *, struct socket *);
+int config_setsock(struct gotwebd *, struct socket *, uid_t, gid_t);
 int config_getsock(struct gotwebd *, struct imsg *);
 int config_setfd(struct gotwebd *);
 int config_getfd(struct gotwebd *, struct imsg *);
