@@ -209,6 +209,7 @@ struct recv_painted_commit_arg {
 	struct got_ratelimit *rl;
 	got_cancel_cb cancel_cb;
 	void *cancel_arg;
+	struct got_object_qid **qid0;
 };
 
 static const struct got_error *
@@ -216,31 +217,36 @@ recv_painted_commit(void *arg, struct got_object_id *id, intptr_t color)
 {
 	const struct got_error *err = NULL;
 	struct recv_painted_commit_arg *a = arg;
-	struct got_object_qid *qid, *tmp;
+	struct got_object_qid *qid;
 
 	if (a->cancel_cb) {
 		err = a->cancel_cb(a->cancel_arg);
 		if (err)
 			return err;
 	}
-
 	switch (color) {
 	case COLOR_KEEP:
-		err = got_object_idset_add(a->keep, id, NULL);
-		if (err)
-			return err;
-		(*a->ncolored)++;
+		if (!got_object_idset_contains(a->keep, id)) {
+			err = got_object_idset_add(a->keep, id, NULL);
+			if (err)
+				return err;
+			(*a->ncolored)++;
+		}
 		break;
 	case COLOR_DROP:
-		err = got_object_idset_add(a->drop, id, NULL);
-		if (err)
-			return err;
-		(*a->ncolored)++;
+		if (!got_object_idset_contains(a->drop, id)) {
+			err = got_object_idset_add(a->drop, id, NULL);
+			if (err)
+				return err;
+			(*a->ncolored)++;
+		}
 		break;
 	case COLOR_SKIP:
-		err = got_object_idset_add(a->skip, id, NULL);
-		if (err)
-			return err;
+		if (!got_object_idset_contains(a->skip, id)) {
+			err = got_object_idset_add(a->skip, id, NULL);
+			if (err)
+				return err;
+		}
 		break;
 	default:
 		/* should not happen */
@@ -248,15 +254,18 @@ recv_painted_commit(void *arg, struct got_object_id *id, intptr_t color)
 		    "%s invalid commit color %"PRIdPTR, __func__, color);
 	}
 
-	STAILQ_FOREACH_SAFE(qid, a->ids, entry, tmp) {
+	STAILQ_FOREACH(qid, a->ids, entry) {
+		int ocolor;
 		if (got_object_id_cmp(&qid->id, id) != 0)
 			continue;
-		STAILQ_REMOVE(a->ids, qid, got_object_qid, entry);
-		color = (intptr_t)qid->data;
-		got_object_qid_free(qid);
-		(*a->nqueued)--;
-		if (color == COLOR_SKIP)
-			(*a->nskip)--;
+		ocolor = (intptr_t)qid->data;
+		if (ocolor != color) {
+			got_pack_paint_commit(qid, color);
+			if (ocolor == COLOR_SKIP)
+				(*a->nskip)--;
+			else if (color == COLOR_SKIP)
+				(*a->nskip)++;
+		}
 		break;
 	}
 
@@ -265,9 +274,9 @@ recv_painted_commit(void *arg, struct got_object_id *id, intptr_t color)
 }
 
 static const struct got_error *
-paint_packed_commits(struct got_pack *pack, struct got_object_id *id,
-    int idx, intptr_t color, int *ncolored, int *nqueued, int *nskip,
-    struct got_object_id_queue *ids,
+paint_packed_commits(struct got_object_qid **qid0,
+    struct got_pack *pack, intptr_t color,
+    int *ncolored, int *nqueued, int *nskip, struct got_object_id_queue *ids,
     struct got_object_idset *keep, struct got_object_idset *drop,
     struct got_object_idset *skip, struct got_repository *repo,
     got_pack_progress_cb progress_cb, void *progress_arg,
@@ -280,8 +289,12 @@ paint_packed_commits(struct got_pack *pack, struct got_object_id *id,
 
 	STAILQ_INIT(&next_ids);
 
-	err = got_privsep_send_painting_request(pack->privsep_child->ibuf,
-	    idx, id, color);
+	err = got_privsep_send_painting_request(pack->privsep_child->ibuf);
+	if (err)
+		return err;
+
+	err = got_privsep_send_object_id_queue(pack->privsep_child->ibuf,
+	    ids, *nqueued);
 	if (err)
 		return err;
 
@@ -297,6 +310,7 @@ paint_packed_commits(struct got_pack *pack, struct got_object_id *id,
 	arg.rl = rl;
 	arg.cancel_cb = cancel_cb;
 	arg.cancel_arg = cancel_arg;
+	arg.qid0 = qid0;
 	err = got_privsep_recv_painted_commits(&next_ids,
 	    recv_painted_commit, &arg, pack->privsep_child->ibuf);
 	if (err)
@@ -327,7 +341,10 @@ paint_packed_commits(struct got_pack *pack, struct got_object_id *id,
 		qid = STAILQ_FIRST(&next_ids);
 		STAILQ_REMOVE_HEAD(&next_ids, entry);
 		got_pack_paint_commit(qid, color);
-		STAILQ_INSERT_TAIL(ids, qid, entry);
+		if (color == COLOR_KEEP)
+			STAILQ_INSERT_TAIL(ids, qid, entry);
+		else
+			STAILQ_INSERT_HEAD(ids, qid, entry);
 		(*nqueued)++;
 		if (color == COLOR_SKIP)
 			(*nskip)++;
@@ -336,6 +353,32 @@ paint_packed_commits(struct got_pack *pack, struct got_object_id *id,
 	return err;
 }
 
+static const struct got_error *
+pin_pack_for_commit_painting(struct got_pack **pack,
+    struct got_packidx **packidx, struct got_object_id_queue *ids,
+    struct got_repository *repo)
+{
+	const struct got_error *err;
+
+	err = got_pack_find_pack_for_commit_painting(packidx, ids, repo);
+	if (err || *packidx == NULL) {
+		*pack = NULL;
+		return err;
+	}
+
+	err = got_pack_cache_pack_for_packidx(pack, *packidx, repo);
+	if (err)
+		return err;
+
+	if ((*pack)->privsep_child == NULL) {
+		err = got_pack_start_privsep_child(*pack, *packidx);
+		if (err)
+			return err;
+	}
+
+	return got_repo_pin_pack(repo, *packidx, *pack);
+}
+
 const struct got_error *
 got_pack_paint_commits(int *ncolored, struct got_object_id_queue *ids, int nids,
     struct got_object_idset *keep, struct got_object_idset *drop,
@@ -352,6 +395,10 @@ got_pack_paint_commits(int *ncolored, struct got_object_id_queue *ids, int nids,
 	int nqueued = nids, nskip = 0;
 	int idx;
 
+	err = pin_pack_for_commit_painting(&pack, &packidx, ids, repo);
+	if (err)
+		return err;
+
 	while (!STAILQ_EMPTY(ids) && nskip != nqueued) {
 		intptr_t color;
 
@@ -361,10 +408,65 @@ got_pack_paint_commits(int *ncolored, struct got_object_id_queue *ids, int nids,
 				break;
 		}
 
+		/* Pinned pack may have moved to different cache slot. */
+		pack = got_repo_get_pinned_pack(repo);
+		if (pack == NULL) {
+			err = pin_pack_for_commit_painting(&pack, &packidx,
+			    ids, repo);
+			if (err)
+				break;
+		}
+
 		qid = STAILQ_FIRST(ids);
+		color = (intptr_t)qid->data;
+
+		/* Use the fast path if we have a suitable pack file. */
+		if (packidx && pack) {
+			idx = got_packidx_get_object_idx(packidx, &qid->id);
+			if (idx != -1) {
+				err = got_privsep_init_commit_painting(
+				    pack->privsep_child->ibuf);
+				if (err)
+					goto done;
+				err = send_idset(pack->privsep_child->ibuf,
+				    keep);
+				if (err)
+					goto done;
+				err = send_idset(pack->privsep_child->ibuf,
+				    drop);
+				if (err)
+					goto done;
+				err = send_idset(pack->privsep_child->ibuf,
+				    skip);
+				if (err)
+					goto done;
+				err = paint_packed_commits(&qid, pack,
+				    color, ncolored, &nqueued, &nskip,
+				    ids, keep, drop, skip, repo,
+				    progress_cb, progress_arg, rl,
+				    cancel_cb, cancel_arg);
+				if (err && qid == NULL)
+					goto done;
+				if (qid) {
+					STAILQ_REMOVE(ids, qid,
+					    got_object_qid, entry);
+					nqueued--;
+					got_object_qid_free(qid);
+					qid = NULL;
+					if (err)
+						goto done;
+				}
+				continue;
+			}
+		}
+
 		STAILQ_REMOVE_HEAD(ids, entry);
 		nqueued--;
-		color = (intptr_t)qid->data;
+
+		got_repo_unpin_pack(repo);
+		pack = NULL;
+		packidx = NULL;
+
 		if (color == COLOR_SKIP)
 			nskip--;
 
@@ -386,31 +488,16 @@ got_pack_paint_commits(int *ncolored, struct got_object_id_queue *ids, int nids,
 			continue;
 		}
 
-		/* Pinned pack may have moved to different cache slot. */
-		pack = got_repo_get_pinned_pack(repo);
-
-		if (packidx && pack) {
-			idx = got_packidx_get_object_idx(packidx, &qid->id);
-			if (idx != -1) {
-				err = paint_packed_commits(pack, &qid->id,
-				    idx, color, ncolored, &nqueued, &nskip,
-				    ids, keep, drop, skip, repo,
-				    progress_cb, progress_arg, rl,
-				    cancel_cb, cancel_arg);
-				if (err)
-					break;
-				got_object_qid_free(qid);
-				qid = NULL;
-				continue;
-			}
-		}
-
 		switch (color) {
 		case COLOR_KEEP:
 			if (got_object_idset_contains(drop, &qid->id)) {
 				err = got_pack_paint_commit(qid, COLOR_SKIP);
 				if (err)
 					goto done;
+				err = got_object_idset_add(skip, &qid->id,
+				    NULL);
+				if (err)
+					goto done;
 			} else
 				(*ncolored)++;
 			err = got_object_idset_add(keep, &qid->id, NULL);
@@ -422,6 +509,10 @@ got_pack_paint_commits(int *ncolored, struct got_object_id_queue *ids, int nids,
 				err = got_pack_paint_commit(qid, COLOR_SKIP);
 				if (err)
 					goto done;
+				err = got_object_idset_add(skip, &qid->id,
+				    NULL);
+				if (err)
+					goto done;
 			} else
 				(*ncolored)++;
 			err = got_object_idset_add(drop, &qid->id, NULL);
@@ -461,51 +552,13 @@ got_pack_paint_commits(int *ncolored, struct got_object_id_queue *ids, int nids,
 				err = got_pack_queue_commit_id(ids, &pid->id,
 				    color, repo);
 				if (err)
-					break;
+					goto done;
 				nqueued++;
 				if (color == COLOR_SKIP)
 					nskip++;
 			}
 		}
 
-		if (pack == NULL && (commit->flags & GOT_COMMIT_FLAG_PACKED)) {
-			if (packidx == NULL) {
-				err = got_pack_find_pack_for_commit_painting(
-				    &packidx, ids, nqueued, repo);
-				if (err)
-					goto done;
-			}
-			if (packidx != NULL) {
-				err = got_pack_cache_pack_for_packidx(&pack,
-				    packidx, repo);
-				if (err)
-					goto done;
-				if (pack->privsep_child == NULL) {
-					err = got_pack_start_privsep_child(
-					    pack, packidx);
-					if (err)
-						goto done;
-				}
-				err = got_privsep_init_commit_painting(
-				    pack->privsep_child->ibuf);
-				if (err)
-					goto done;
-				err = send_idset(pack->privsep_child->ibuf,
-				    keep);
-				if (err)
-					goto done;
-				err = send_idset(pack->privsep_child->ibuf, drop);
-				if (err)
-					goto done;
-				err = send_idset(pack->privsep_child->ibuf, skip);
-				if (err)
-					goto done;
-				err = got_repo_pin_pack(repo, packidx, pack);
-				if (err)
-					goto done;
-			}
-		}
-
 		got_object_commit_close(commit);
 		commit = NULL;
 
